@@ -5,7 +5,6 @@
 from __future__ import (absolute_import, division, print_function,
                         with_statement)
 
-from   collections              import Callable
 from   contextlib               import contextmanager
 import errno
 from   functools                import wraps
@@ -19,6 +18,11 @@ from   types                    import CodeType, FrameType, TracebackType
 import six
 from   six.moves                import builtins
 
+if six.PY3:
+    from   collections.abc          import Callable
+else:
+    from   collections              import Callable
+
 from   pyflyby._file            import Filename
 
 
@@ -29,6 +33,27 @@ and if so what.
 _waiting_for_debugger = None
 
 
+_ORIG_SYS_EXCEPTHOOK = sys.excepthook
+
+def _reset_excepthook():
+    if _ORIG_SYS_EXCEPTHOOK:
+        sys.excepthook = _ORIG_SYS_EXCEPTHOOK
+        return True
+    return False
+
+
+def _override_excepthook(hook):
+    """
+    Override sys.excepthook with `hook` but also support resetting.
+
+    Users should call this function instead of directly overiding
+    sys.excepthook. This is helpful in resetting sys.excepthook in certain cases.
+    """
+    global _ORIG_SYS_EXCEPTHOOK
+    _ORIG_SYS_EXCEPTHOOK = hook
+    sys.excepthook = hook
+
+
 class _NoTtyError(Exception):
     pass
 
@@ -570,6 +595,17 @@ def wait_for_debugger_to_attach(arg, mailto=None, background=False, timeout=8640
     else:
         originalpid = None
     try:
+        # Reset the exception hook after the first exception.
+        #
+        # In case the code injected by the remote client causes some error in
+        # the debugged process, another email is sent for the new exception. This can
+        # lead to an infinite loop of sending mail for each successive exceptions
+        # everytime a remote client tries to connect. Our process might never get
+        # a chance to exit and the remote client might just hang.
+        #
+        if not _reset_excepthook():
+            raise ValueError("Couldn't reset sys.excepthook. Aborting remote "
+                             "debugging.")
         # Send email.
         _send_email_with_attach_instructions(arg, mailto, originalpid=originalpid)
         # Sleep until the debugger to attaches.
@@ -775,7 +811,7 @@ def enable_exception_handler_debugger():
     Enable ``sys.excepthook = debugger`` so that we automatically enter
     the debugger upon uncaught exceptions.
     '''
-    sys.excepthook = debugger
+    _override_excepthook(debugger)
 
 
 # Handle SIGTERM with traceback+exit.
@@ -903,19 +939,19 @@ def _escape_for_gdb(string):
     return ''.join(result)
 
 
-_memoized_dev_null_w = None
-def _dev_null_w():
+_memoized_dev_null = None
+def _dev_null():
     """
-    Return a file object opened for writing to /dev/null.
+    Return a file object opened for reading/writing to /dev/null.
     Memoized.
 
     :rtype:
       ``file``
     """
-    global _memoized_dev_null_w
-    if _memoized_dev_null_w is None:
-        _memoized_dev_null_w = open("/dev/null", 'w')
-    return _memoized_dev_null_w
+    global _memoized_dev_null
+    if _memoized_dev_null is None:
+        _memoized_dev_null = open("/dev/null", 'w+')
+    return _memoized_dev_null
 
 
 def inject(pid, statements, wait=True, show_gdb_output=False):
@@ -959,11 +995,20 @@ def inject(pid, statements, wait=True, show_gdb_output=False):
             "pid %s uses executable %s, which does not appear to be python"
             % (pid, python_path))
     # TODO: check that gdb is found and that the version is new enough (7.x)
+    #
+    # A note about --interpreter=mi: mi stands for Machine Interface and it's
+    # the blessed way to control gdb from a pipe, since the output is much
+    # easier to parse than the normal human-oriented output (it is also worth
+    # noting that at the moment we are never parsig the output, but it's still
+    # a good practice to use --interpreter=mi).
     command = (
-        ['gdb', str(python_path), '-p', str(pid), '-batch']
+        ['gdb', str(python_path), '-p', str(pid), '-batch', '--interpreter=mi']
         + [ '-eval-command=call %s' % (c,) for c in gdb_commands ])
-    output = None if show_gdb_output else _dev_null_w()
-    process = subprocess.Popen(command, stdout=output, stderr=output)
+    output = None if show_gdb_output else _dev_null()
+    process = subprocess.Popen(command,
+                               stdin=_dev_null(),
+                               stdout=output,
+                               stderr=output)
     if wait:
         retcode = process.wait()
         if retcode:
@@ -974,6 +1019,28 @@ def inject(pid, statements, wait=True, show_gdb_output=False):
         return process.pid
 
 
+import tty
+
+
+# Copy of tty.setraw that does not set ISIG,
+# in order to keep CTRL-C sending Keybord Interrupt.
+def setraw_but_sigint(fd, when=tty.TCSAFLUSH):
+    """Put terminal into a raw mode."""
+    mode = tty.tcgetattr(fd)
+    mode[tty.IFLAG] = mode[tty.IFLAG] & ~(
+        tty.BRKINT | tty.ICRNL | tty.INPCK | tty.ISTRIP | tty.IXON
+    )
+    mode[tty.OFLAG] = mode[tty.OFLAG] & ~(tty.OPOST)
+    mode[tty.CFLAG] = mode[tty.CFLAG] & ~(tty.CSIZE | tty.PARENB)
+    mode[tty.CFLAG] = mode[tty.CFLAG] | tty.CS8
+    mode[tty.LFLAG] = mode[tty.LFLAG] & ~(
+        tty.ECHO | tty.ICANON | tty.IEXTEN
+    )  # NOT ISIG HERE.
+    mode[tty.CC][tty.VMIN] = 1
+    mode[tty.CC][tty.VTIME] = 0
+    tty.tcsetattr(fd, when, mode)
+
+
 class Pty(object):
     def __init__(self):
         import pty
@@ -985,12 +1052,14 @@ class Pty(object):
         import pty
         try:
             mode = tty.tcgetattr(pty.STDIN_FILENO)
-            tty.setraw(pty.STDIN_FILENO)
+            setraw_but_sigint(pty.STDIN_FILENO)
             restore = True
         except tty.error:
             restore = False
         try:
             pty._copy(self.master_fd)
+        except KeyboardInterrupt:
+            print('^C\r') # we need the \r because we are still in raw mode
         finally:
             if restore:
                 tty.tcsetattr(pty.STDIN_FILENO, tty.TCSAFLUSH, mode)
@@ -1057,40 +1126,71 @@ def attach_debugger(pid):
     # Inject a call to 'debugger()' into target process.
     # Set on_continue to signal ourselves that we're done.
     on_continue = "lambda: __import__('os').kill(%d, %d)" % (os.getpid(), signal.SIGUSR1)
-    gdb_pid = inject(pid, [
-            "__import__('sys').path.insert(0, %r)" % (pyflyby_lib_path,),
-            ("__import__('pyflyby').debugger(tty=%r, on_continue=%s)"
-             % (terminal.ttyname, on_continue)),
-            ], wait=False)
+
+    # Use Python import machinery to import pyflyby from its directory.
+    #
+    # Adding the path to sys.path might have side effects. For e.g., a package
+    # with the same name as a built-in module could exist in `pyflyby_dir`.
+    # Adding `pyflyby_dir` to sys.path will make the package get imported from
+    # `pyflyby_dir` instead of deferring this decision to the user Python
+    # environment.
+    #
+    # As a concrete example, `typing` module is a package as well a built-in
+    # module from Python version >= 3.5
+    if six.PY2:
+        statements = [(
+            "location = __import__('imp').find_module('pyflyby', ['{pyflyby_dir}'])"
+            .format(pyflyby_dir=pyflyby_lib_path)),
+            "pyflyby = __import__('pkgutil').ImpLoader('pyflyby', *location).load_module('pyflyby')"
+            ]
+    else:
+        statements = [
+            "loader = __import__('importlib').machinery.PathFinder.find_module("
+            "fullname='pyflyby', path=['{pyflyby_dir}'])".format(
+                pyflyby_dir=pyflyby_lib_path),
+            "pyflyby = loader.load_module('pyflyby')"
+        ]
+    statements.append(
+        ("pyflyby.debugger(tty=%r, on_continue=%s)"
+         % (terminal.ttyname, on_continue))
+        )
+
+    gdb_pid = inject(pid, statements=";".join(statements), wait=False)
     # Fork a watchdog process to make sure we exit if the target process or
     # gdb process exits, and make sure the gdb process exits if we exit.
     parent_pid = os.getpid()
     watchdog_pid = os.fork()
     if watchdog_pid == 0:
         while True:
-            if not process_exists(gdb_pid):
-                kill_process(
-                    parent_pid,
-                    [(signal.SIGUSR1, 5), (signal.SIGTERM, 15),
-                     (signal.SIGKILL, 60)])
-                break
-            if not process_exists(pid):
-                start_time = time.time()
-                os.kill(parent_pid, signal.SIGUSR1)
-                kill_process(
-                    gdb_pid,
-                    [(0, 5), (signal.SIGTERM, 15), (signal.SIGKILL, 60)])
-                kill_process(
-                    parent_pid,
-                    [(0, (5 + time.time() - start_time)),
-                     (signal.SIGTERM, 15), (signal.SIGKILL, 60)])
-                break
-            if not process_exists(parent_pid):
-                kill_process(
-                    gdb_pid,
-                    [(0, 5), (signal.SIGTERM, 15), (signal.SIGKILL, 60)])
-                break
-            time.sleep(0.1)
+            try:
+                if not process_exists(gdb_pid):
+                    kill_process(
+                        parent_pid,
+                        [(signal.SIGUSR1, 5), (signal.SIGTERM, 15),
+                         (signal.SIGKILL, 60)])
+                    break
+                if not process_exists(pid):
+                    start_time = time.time()
+                    os.kill(parent_pid, signal.SIGUSR1)
+                    kill_process(
+                        gdb_pid,
+                        [(0, 5), (signal.SIGTERM, 15), (signal.SIGKILL, 60)])
+                    kill_process(
+                        parent_pid,
+                        [(0, (5 + time.time() - start_time)),
+                         (signal.SIGTERM, 15), (signal.SIGKILL, 60)])
+                    break
+                if not process_exists(parent_pid):
+                    kill_process(
+                        gdb_pid,
+                        [(0, 5), (signal.SIGTERM, 15), (signal.SIGKILL, 60)])
+                    break
+                time.sleep(0.1)
+            except KeyboardInterrupt:
+                # if the user pressed CTRL-C the parent process is about to
+                # die, so we will detect the death in the next iteration of
+                # the loop and exit cleanly after killing also gdb
+                pass
         os._exit(0)
     # Communicate with pseudo tty.
     try:
