@@ -1210,6 +1210,17 @@ def test_find_missing_imports_exception_3():
     expected = ['SomeException']
     assert expected == result
 
+
+def test_find_missing_imports_tuple_ellipsis_type_1():
+    code = dedent("""
+    tuple[Foo, ..., Bar]
+    """)
+    result = find_missing_imports(code, [{}])
+    result = _dilist2strlist(result)
+    expected = ['Bar', 'Foo']
+    assert expected == result
+
+
 # Only Python 3.8 includes type comments in the ast, so we only support this
 # there (see issue #31).
 @pytest.mark.skipif(
@@ -1296,10 +1307,11 @@ def test_scan_for_import_issues_class_subclass_imported_class_1():
     assert unused == []
 
 
-# This is currently buggy.
-# The problem is that we postpone the check for C1.base = m1.C1, and by the
-# time we check it, we've already replaced the thing in the scope.
-@pytest.mark.xfail
+# This is now an XPASS, let's remove the xfail to see if it fails only on some CI version.
+# # This is currently buggy.
+# # The problem is that we postpone the check for C1.base = m1.C1, and by the
+# # time we check it, we've already replaced the thing in the scope.
+# @pytest.mark.xfail
 def test_scan_for_import_issues_class_subclass_imported_class_in_func_1():
     code = dedent("""
         def f1():
@@ -1459,8 +1471,8 @@ def test_scan_for_import_issues_comprehension_attribute_missing_1():
     assert missing == [(3, DottedIdentifier('xx'))]
     assert unused == []
 
-
-@pytest.mark.xfail
+# this is now an XPASS, check that it's passing everywhere in CI
+# @pytest.mark.xfail
 def test_scan_for_import_issues_comprehension_attribute_complex_1():
     code = dedent("""
         dd = []
